from tuyapy.devices.base import TuyaDevice


class TuyaCover(TuyaDevice):

    def state(self):
        state = self.data.get('state')
        return state

    def open_cover(self):
        """Open the cover."""
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '1'})

    def close_cover(self):
        """Close cover."""
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '0'})

    def stop_cover(self):
        """Stop the cover."""
        self.api.device_control(self.obj_id, 'startStop', {'value': '0'})

    def support_stop(self):
        support  = self.data.get('support_stop')
        if support is None:
            return False
        return support








from tuyapy.devices.base import TuyaDevice


class TuyaScene(TuyaDevice):

    def available(self):
        return True

    def activate(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '1'})

    def update(self):
        return True











from tuyapy.devices.climate import TuyaClimate
from tuyapy.devices.light import TuyaLight
from tuyapy.devices.scene import TuyaScene
from tuyapy.devices.fan import TuyaFanDevice
from tuyapy.devices.cover import TuyaCover
from tuyapy.devices.lock import TuyaLock
from tuyapy.devices.switch import TuyaSwitch

def get_tuya_device(data, api):
    dev_type = data.get('dev_type')
    devices = []

    if dev_type == 'light':
        devices.append(TuyaLight(data, api))
    elif dev_type == 'climate':
        devices.append(TuyaClimate(data, api))
    elif dev_type == 'scene':
        devices.append(TuyaScene(data, api))
    elif dev_type == 'fan':
        devices.append(TuyaFanDevice(data, api))
    elif dev_type == 'cover':
        devices.append(TuyaCover(data, api))
    elif dev_type == 'lock':
        devices.append(TuyaLock(data, api))
    elif dev_type == 'switch':
        devices.append(TuyaSwitch(data, api))
    return devices






from tuyapy.devices.base import TuyaDevice


class TuyaLock(TuyaDevice):

    def state(self):
        state = self.data.get('state')
        if state == 'true':
            return True
        elif state == 'false':
            return False
        else:
            return None



from tuyapy.devices.base import TuyaDevice


class TuyaSwitch(TuyaDevice):

    def state(self):
        state = self.data.get('state')
        if state is None:
            return None
        return state

    def turn_on(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '1'})

    def turn_off(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '0'})



import json
import logging
import requests
import time

from tuyapy.devices.factory import get_tuya_device


TUYACLOUDURL = "https://px1.tuya{}.com"
DEFAULTREGION = 'us'

REFRESHTIME = 60 * 60 * 12

_LOGGER = logging.getLogger(__name__)


class TuyaSession:

    username = ''
    password = ''
    countryCode = ''
    bizType = ''
    accessToken = ''
    refreshToken = ''
    expireTime = 0
    devices = []
    region = DEFAULTREGION


SESSION = TuyaSession()


class TuyaApi:

    def init(self, username, password, countryCode, bizType=''):
        SESSION.username = username
        SESSION.password = password
        SESSION.countryCode = countryCode
        SESSION.bizType = bizType

        if username is None or password is None:
            return None
        else:
            self.get_access_token()
            self.discover_devices()
            return SESSION.devices

    def get_access_token(self):
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }# have a test
        response = requests.post(
            (TUYACLOUDURL+'/homeassistant/auth.do').format(SESSION.region),
            data = {
                'userName': SESSION.username,
                'password': SESSION.password,
                'countryCode': SESSION.countryCode,
                'bizType':SESSION.bizType,
                'from': 'tuya'
            },
        )
        response_json = response.json()
        if response_json.get('responseStatus') == 'error':
            message = response_json.get('errorMsg')
            if message == 'error':
                raise TuyaAPIException("get access token failed")
            else:
                raise TuyaAPIException(message)

        SESSION.accessToken = response_json.get('access_token')
        SESSION.refreshToken = response_json.get('refresh_token')
        SESSION.expireTime = int(time.time()) + response_json.get('expires_in')
        areaCode = SESSION.accessToken[0: 2]
        if areaCode == 'AY':
            SESSION.region = 'cn'
        elif areaCode == 'EU':
            SESSION.region = 'eu'
        else:
            SESSION.region = 'us'

    def check_access_token(self):
        if SESSION.username == '' or SESSION.password == '':
            raise TuyaAPIException("can not find username or password")
            return
        if SESSION.accessToken == '' or SESSION.refreshToken == '':
            self.get_access_token()
        elif SESSION.expireTime <= REFRESHTIME + int(time.time()):
            self.refresh_access_token()

    def refresh_access_token(self):
        data = "grant_type=refresh_token&refresh_token="+SESSION.refreshToken
        response = requests.get(
            (TUYACLOUDURL+"homeassistant/access.do").format(SESSION.region)
            + "?" + data)
        response_json = response.json()
        if response_json.get('responseStatus') == 'error':
            raise TuyaAPIException('refresh token failed')

        SESSION.accessToken = response_json.get('access_token')
        SESSION.refreshToken = response_json.get('refresh_token')
        SESSION.expireTime = int(time.time()) + response_json.get('expires_in')

    def poll_devices_update(self):
        self.check_access_token()
        return self.discover_devices()

    def discover_devices(self):
        response = self._request('Discovery', 'discovery')
        if response['header']['code'] == 'SUCCESS':
            SESSION.devices = []
            for device in response['payload']['devices']:
                SESSION.devices.extend(get_tuya_device(device, self))
            return response['payload']['devices']
        return None

    def get_devices_by_type(self, dev_type):
        device_list = []
        for device in SESSION.devices:
            if device.dev_type() == dev_type:
                device_list.append(device)

    def get_all_devices(self):
        return SESSION.devices

    def get_device_by_id(self, dev_id):
        for device in SESSION.devices:
            if device.object_id() == dev_id:
                return device
        return None

    def device_control(self, devId, action, param=None, namespace='control'):
        if param is None:
            param = {}
        response = self._request(action, namespace, devId, param)
        if response['header']['code'] == 'SUCCESS':
            success = True
        else:
            success = False
        return success,response

    def _request(self, name, namespace, devId=None, payload={}):
        headers = {
            'Content-Type': 'application/json'
        }
        header = {
            'name': name,
            'namespace': namespace,
            'payloadVersion': 1,
        }
        payload['accessToken'] = SESSION.accessToken
        if namespace != 'discovery':
            payload["devId"] = devId
        data = {
            'header': header,
            'payload': payload
        }
        response = requests.post(
            (TUYACLOUDURL+'/homeassistant/skill').format(SESSION.region),
            json = data
        )
        response_json = response.json()
        if response_json['header']['code'] != 'SUCCESS':
            _LOGGER.debug("control device error, error code is " +
                response_json['header']['code'])
        return response_json


class TuyaAPIException(Exception):
    pass



import time


class TuyaDevice(object):
    def __init__(self, data, api):
        self.api = api
        self.data = data.get('data')
        self.obj_id = data.get('id')
        self.obj_type = data.get('ha_type')
        self.obj_name = data.get('name')
        self.dev_type = data.get('dev_type')
        self.icon = data.get('icon')

    def name(self):
        return self.obj_name

    def state(self):
        state = self.data.get('state')
        if state == 'true':
            return True
        else:
            return False

    def device_type(self):
        return self.dev_type

    def object_id(self):
        return self.obj_id

    def object_type(self):
        return self.obj_type

    def available(self):
        return self.data.get('online')

    def iconurl(self):
        return self.icon

    def update(self):
        """Avoid get cache value after control."""
        time.sleep(0.5)
        success, response = self.api.device_control(
            self.obj_id, 'QueryDevice', namespace='query')
        if success:
            self.data = response['payload']['data']
            return True
        return
        











from tuyapy.devices.base import TuyaDevice


class TuyaFanDevice(TuyaDevice):
    
    def state(self):
        state = self.data.get('state')
        if state == 'true':
            return True
        else:
            return False

    def speed(self):
        return self.data.get('speed')

    def speed_list(self):
        speed_list = []
        speed_level = self.data.get('speed_level')
        for i in range(speed_level):
            speed_list.append(str(i+1))
        return speed_list

    def oscillating(self):
        return self.data.get('direction')

    def set_speed(self, speed):
        self.api.device_control(self.obj_id, 'windSpeedSet', {'value': speed})

    def oscillate(self, oscillating):
        if oscillating:
            command = 'swingOpen'
        else:
            command = 'swingClose'
        self.api.device_control(self.obj_id, command)

    def turn_on(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '1'})

    def turn_off(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '0'})

    def support_oscillate(self):
        if self.oscillating() is None:
            return False
        else:
            return True

    def support_direction(self):
        return False






from tuyapy.devices.base import TuyaDevice


class TuyaClimate(TuyaDevice):

    def temperature_unit(self):
        return self.data.get('temp_unit')

    def current_humidity(self):
        pass

    def target_humidity(self):
        pass

    def current_operation(self):
        return self.data.get('mode')

    def operation_list(self):
        return self.data.get('support_mode')

    def current_temperature(self):
        return self.data.get('current_temperature')

    def target_temperature(self):
        return self.data.get('temperature')

    def target_temperature_step(self):
        return 1

    def current_fan_mode(self):
        """Return the fan setting."""
        fan_speed = self.data.get('windspeed')
        if fan_speed is None:
            return None
        if fan_speed == '1':
            return 'low'
        elif fan_speed == '2':
            return 'medium'
        elif fan_speed == '3':
            return 'high'
        return fan_speed

    def fan_list(self):
        """Return the list of available fan modes."""
        return ['low', 'medium', 'high']

    def current_swing_mode(self):
        """Return the fan setting."""
        return None

    def swing_list(self):
        """Return the list of available swing modes."""
        return None

    def min_temp(self):
        return self.data.get('min_temper')

    def max_temp(self):
        return self.data.get('max_temper')

    def min_humidity(self):
        pass

    def max_humidity(self):
        pass

    def set_temperature(self, temperature):
        """Set new target temperature."""
        self.api.device_control(self.obj_id, 'temperatureSet', 
            {'value': int(temperature)})
        
    def set_humidity(self, humidity):
        """Set new target humidity."""
        raise NotImplementedError()

    def set_fan_mode(self, fan_mode):
        """Set new target fan mode."""
        self.api.device_control(self.obj_id, 'windSpeedSet',
            {'value': fan_mode})

    def set_operation_mode(self, operation_mode):
        """Set new target operation mode."""
        self.api.device_control(self.obj_id, 'modeSet', 
            {'value': operation_mode})

    def set_swing_mode(self, swing_mode):
        """Set new target swing operation."""
        raise NotImplementedError()

    def support_target_temperature(self):
        if self.data.get('temperature') is not None:
            return True
        else:
            return False

    def support_mode(self):
        if self.data.get('mode') is not None:
            return True
        else:
            return False

    def support_wind_speed(self):
        if self.data.get('windspeed') is not None:
            return True
        else:
            return False

    def support_humidity(self):
        if self.data.get('humidity') is not None:
            return True
        else:
            return False

    def turn_on(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '1'})

    def turn_off(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '0'})





import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="tuyapy",
    version="0.1.4",
    author="Tuya Inc.",
    author_email="tuyasmart@tuya.com",
    description="A Python library for Tuya Device with Python 3+",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from tuyapy.devices.base import TuyaDevice


class TuyaLight(TuyaDevice):

    def state(self):
        state = self.data.get('state')
        if state == 'true':
            return True
        else:
            return False

    def brightness(self):
        work_mode = self.data.get('color_mode')
        if work_mode == 'colour':
            brightness = int(self.data.get('color').get('brightness') * 255 / 100)
        else:
            brightness = self.data.get('brightness')
        return brightness

    def _set_brightness(self, brightness):
        work_mode = self.data.get('color_mode')
        if work_mode == 'colour':
            self.data['color']['brightness'] = brightness
        else:
            self.data['brightness'] = brightness

    def support_color(self):
        if self.data.get('color') is None:
            return False
        else:
            return True

    def support_color_temp(self):
        if self.data.get('color_temp') is None:
            return False
        else:
            return True

    def hs_color(self):
        if self.data.get('color') is None:
            return None
        else:
            work_mode = self.data.get('color_mode')
            if work_mode == 'colour':
                color = self.data.get('color')
                return color.get('hue'), color.get('saturation')
            else:
                return 0.0, 0.0

    def color_temp(self):
        if self.data.get('color_temp') is None:
            return None
        else:
            return self.data.get('color_temp')

    def min_color_temp(self):
        return 10000

    def max_color_temp(self):
        return 1000

    def turn_on(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '1'})

    def turn_off(self):
        self.api.device_control(self.obj_id, 'turnOnOff', {'value': '0'})

    def set_brightness(self, brightness):
        """Set the brightness(0-255) of light."""
        value = int(brightness * 100 / 255)
        self.api.device_control(self.obj_id, 'brightnessSet', {'value': value})

    def set_color(self, color):
        """Set the color of light."""
        hsv_color = {}
        hsv_color['hue'] = color[0]
        hsv_color['saturation'] = color[1]/100
        if (len(color) < 3):
            hsv_color['brightness'] = int(self.brightness()) / 255.0
        else:
            hsv_color['brightness'] = color[2]
        # color white
        if hsv_color['saturation'] == 0:
            hsv_color['hue'] = 0
        self.api.device_control(self.obj_id, 'colorSet', {'color': hsv_color})

    def set_color_temp(self, color_temp):
        self.api.device_control(self.obj_id, 'colorTemperatureSet', 
            {'value': color_temp})
